class Sterling:
    def __init__(self, params):
        self.url = params["base_url"]
        self.headers = {
            "Sandbox-Key": params['Sandbox-Key'],
            "Ocp-Apim-Subscription-Key": params['Ocp-Apim-Subscription-Key'],
            "Ocp-Apim-Trace": params['Ocp-Apim-Trace'],
            "Appid": params['Appid'],
            "Content-Type": params['Content-Type'],
            "ipval": params['ipval']
        }


def error(e):
    if e == 400:
        data = "Bad Request"
    elif e == 401:
        data = "Unauthorized. Please check your credentials " + str(e)
    elif e == 503:
        data = "Service temporarily unavailable " + str(e)
    elif e == 500:
        data = "Error executing methodName please raise an issue on github "+ str(e)
    else:
        data = "Error"
    return data



import base64
import binascii
import json
from Crypto.Cipher import AES
from Crypto.Util.Padding import pad, unpad


class Crypt:
    def __init__(self):
        self.enc_dec_method = 'utf-8'

    def encrypt(self, str_to_enc, key, iv):
        try:
            aes_obj = AES.new(key.encode(self.enc_dec_method), AES.MODE_CBC, iv.encode(self.enc_dec_method))
            text= json.dumps(str_to_enc)
            hx_enc = aes_obj.encrypt(pad(text.encode(self.enc_dec_method), 16))
            enc = binascii.hexlify(hx_enc).decode(self.enc_dec_method)
            return enc

        except ValueError as value_error:
            if value_error.args[0] == 'IV must be 16 bytes long':
                raise ValueError('Encryption Error: IV key must be 16 characters long')
            elif value_error.args[0] == 'AES key must be either 16, 24, or 32 bytes long':
                raise ValueError('Encryption Error: Encryption key must be either 16, 24, or 32 characters long')
            else:
                raise ValueError(value_error)

    def decrypt(self, enc_str, key, iv):
        try:
            raw = binascii.unhexlify(enc_str)
            aes_obj = AES.new(key.encode(self.enc_dec_method), AES.MODE_CBC, iv.encode(self.enc_dec_method))
            mret = unpad(aes_obj.decrypt(raw), 16)
            dec = mret.decode(self.enc_dec_method)
            return json.loads(dec)

        except ValueError as value_error:
            if value_error.args[0] == 'IV must be 16 bytes long':
                raise ValueError('Decryption Error: IV key must be 16 characters long')
            elif value_error.args[0] == 'AES key must be either 16, 24, or 32 bytes long':
                raise ValueError('Decryption Error: Encryption key must be either 16, 24, or 32 characters long')
            else:
                raise ValueError(value_error)

    def hash(self, text):
        try:
            result = (base64.b64encode(text.encode(self.enc_dec_method))).decode('utf8')
            return result
        except ValueError as value_error:
            if value_error.args[0] == 'A value is required':
                raise ValueError('Hashing Error: A value must be provided for hashing')
            else:
                raise ValueError(value_error)






import unittest
from requests import Session
from unittest.mock import Mock, patch
from sterling.tests.common import responses, body, query, R
from sterling.account import Account
from sterling.transfer import Transfer
from faker import Faker

fake = Faker().random_number()

header = {
    "base_url": "",
    "Sandbox-Key": str(Faker().text()),
    "Ocp-Apim-Subscription-Key": str(Faker().text()),
    "Ocp-Apim-Trace": "true",
    "Appid": str(fake),
    "Content-Type": "application/json",
    "ipval": str(fake)
}


@patch.object(Session, 'request')
def test_enquiry(mock_get):
    data = responses["enquiry"]
    mock_get.return_value = R(responses["enquiry"])
    assert Transfer(header).InterbankNameEnquiry(query) == data, "should return an object"


@patch.object(Session, 'request')
def test_transfer(mock_post):
    data = responses["transfer"]
    mock_post.return_value = R(responses["transfer"])
    assert Account(header).InterbankTransferReq(body) == data, "should return an object"



import datetime
import hashlib
from nibss.utils import crypt as crypting


class Nibss:
    def __init__(self, params):
        self.url = params["base_url"]
        self.key = params["sandbox-key"]
        self.content_type = params["content-type"]
        self.accept = params["accept"]
        self.code = crypting.Crypt().hash(params["Organizationcode"])
        self.sign_meth = 'SHA256'
        auth_string = params["username"] + ':' + params["password"]
        self.auth_header = crypting.Crypt().hash(auth_string)
        datenow = str(datetime.date.today()).split('-')
        date = ''.join(datenow)
        signature_string = params["username"] + str(date) + params["password"]
        self.sign_header = hashlib.sha256(signature_string.encode('utf-8')).hexdigest()

        self.headers = {
            "OrganisationCode": self.code,
            "Sandbox-Key": self.key,
            "Authorization": self.auth_header,
            "SIGNATURE": self.sign_header,
            "SIGNATURE_METH": self.sign_meth,
            "Content-Type": self.content_type,
            "Accept": self.accept
        }
        self.header = {
            "OrganisationCode": self.code,
            "Sandbox-Key": self.key
        }


from requests import request, sessions
from sterling.utils.response_handler import response


def calls(method, url, params, data, header):
    try:
        with sessions.Session() as session:
                res = session.request(method= method, url=url, params=params, data=data, headers= header)
                return response(res)
    except Exception as e:
        return e



import json
from urllib.parse import urljoin
from url import url as BASE_URL
from sterling.Sterling import  Sterling
from sterling.utils.calls import calls


class Account(Sterling):
    def InterbankTransferReq(self, body):
        url = urljoin(BASE_URL(self.url), "/sterling/accountapi/api/Spay/InterbankTransferReq")
        header = self.headers
        apiResponse = calls("post", url, None, json.dumps(body), header)
        return (apiResponse)




from nibss.utils.calls import encrypted_request
from nibss.Nibss import Nibss
from urllib.parse import urljoin
from url import url as BASE_URL


class Bvn(Nibss):
    def verify_single(self, data):
        URL = urljoin(BASE_URL(self.url), "/nibss/bvnr/VerifySingleBVN")
        headers = self.headers
        body = data["body"]
        return encrypted_request(headers, URL, data["Aes_key"], data["Iv_key"], body)

    def verify_multiple(self, data):
        URL = urljoin(BASE_URL(self.url), "/nibss/bvnr/VerifyMultipleBVN")
        headers = self.headers
        body = data["bvns"]
        return encrypted_request(headers, URL, data["Aes_key"], data["Iv_key"], body)

    def bvn_watchlisted(self, data):
        URL = urljoin(BASE_URL(self.url), "/nibss/bvnr/IsBVNWatchlisted")
        headers = self.headers
        body = data["body"]
        return encrypted_request(headers, URL, data["Aes_key"], data["Iv_key"], body)



from .utils.calls import encrypted_request
from nibss.Nibss import Nibss
from urllib.parse import urljoin
from url import url as BASE_URL


class FingerPrint(Nibss):
    def verify_fingerprint(self, data):
        URL = urljoin(BASE_URL(self.url), "/nibss/fp/VerifyFingerPrint")
        headers = self.headers
        body = data["body"]
        return encrypted_request(headers, URL, data["Aes_key"], data["Iv_key"], body)





from nibss.Nibss import Nibss
from urllib.parse import urljoin
from url import url as BASE_URL
from nibss.utils.calls import request


class Credentials(Nibss):
    def reset(self):
        headers = self.header
        URL = urljoin(BASE_URL(self.url), "/nibss/bvnr/Reset")
        return request(headers, URL)



from nibss.utils.crypt import Crypt
from Crypto.Random import random

iv = ''.join(chr(random.randint(0, 23)) for i in range(16))
aes = ''.join(chr(random.randint(0, 23)) for i in range(16))

responses = {
    "single_bvn": {
        'message': 'OK',
        'data': {
            'ResponseCode': '00', 'BVN': '12345678901', 'FirstName': 'Uchenna', 'MiddleName': 'Chijioke',
            'LastName': 'Nwanyanwu', 'DateOfBirth': '22-Oct-1970', 'PhoneNumber': '07033333333',
            'RegistrationDate': '16-Nov-2014', 'EnrollmentBank': '900',
            'EnrollmentBranch': 'Victoria Island', 'WatchListed': 'NO'
        }
    },
    "multiple_bvn": {
        "message": "OK",
        "data": {"ResponseCode": "00", "ValidationResponses": [
            {
                "ResponseCode": "00", "BVN": "12345678901", "FirstName": "Uchenna", "MiddleName": "Innocent",
                "LastName": "Nwanyanwu", "DateOfBirth": "29-Oct-1995", "PhoneNumber": "07033333333",
                "RegistrationDate": "16-Dec-2014", "EnrollmentBank": "900", "EnrollmentBranch": "Victoria Island",
                "WatchListed": "NO"
            },
            {
                "ResponseCode": "00", "BVN": "12345678902", "FirstName": "Wale", "MiddleName": "Joshua",
                "LastName": "Odugbemi", "DateOfBirth": "29-Oct-1996", "PhoneNumber": "07033333334",
                "RegistrationDate": "16-Oct-2014", "EnrollmentBank": "900",
                "EnrollmentBranch": "No. 2 NIBSS Avenue, VI",
                "WatchListed": "YES"
            },
            {
                "ResponseCode": "00", "BVN": "12345678903", "FirstName": "Seun", "MiddleName": "Ogunjimi",
                "LastName": "Isaiah", "DateOfBirth": "29-Oct-1997", "PhoneNumber": "07033333336",
                "RegistrationDate": "16-Sept-2014", "EnrollmentBank": "900", "EnrollmentBranch": "Ikorodu",
                "WatchListed": "NO"
            }]}
    },
    "watchlist": {
        "message": "OK",
        "data": {
            "ResponseCode": "00",
            "BVN": "12345678901",
            "BankCode": "900",
            "Category": "1",
            "WatchListed": "YES"
        }
    },
    "record": {
        "message": "OK",
        "data": {
            "ResponseCode": "00",
            "BVN": "VALID",
            "FirstName": "VALID",
            "LastName": "VALID",
            "MiddleName": "INVALID",
            "AccountNumber": "VALID",
            "BankCode": "VALID"
        }
    },
    "records": {
        'message': 'OK',
        'data': {'ValidationResponses': [
            {
                'ResponseCode': '00',
                'BVN': 'VALID',
                'FirstName': 'VALID',
                'LastName': 'VALID',
                'MiddleName': 'INVALID',
                'AccountNumber': 'VALID',
                'BankCode': 'VALID'},
            {
                'ResponseCode': '00', 'BVN': 'VALID', 'FirstName': 'INVALID', 'LastName': 'VALID',
                'MiddleName': 'INVALID', 'AccountNumber': 'VALID', 'BankCode': 'VALID'
            }
        ]}
    },
    "fingerprint_data": {
        "message": "OK",
        "data": {
            "BVN": "12345678901",
            "ResponseCode": "00"
        }
    }

}

body = {
    "single_bvn": Crypt().encrypt(responses["single_bvn"], aes, iv),
    "multiple_bvn": Crypt().encrypt(responses["multiple_bvn"], aes, iv),
    "watchlist": Crypt().encrypt(responses["watchlist"], aes, iv),
    "record": Crypt().encrypt(responses["record"], aes, iv),
    "records": Crypt().encrypt(responses["records"], aes, iv),
    "fingerprint_data": Crypt().encrypt(responses["fingerprint_data"], aes, iv)
}


class R:
    def __init__(self, text):
        self.status_code = 200
        self.text = text
        self.headers = {'Aes_key': aes, 'Password': "fdgfudkjd", 'Ivkey': iv}



def response(response):
    if response.status_code == 200:
        data = response.text
    elif response.status_code == 400:
        data = "Bad Request"
    elif response == 401:
        data = "Unauthorized. Please check your credentials " + str(response.status_code)
    elif response.status_code == 503:
        data = "Service temporarily unavailable " + str(response.status_code)
    elif response.status_code == 500:
        data = "Error executing methodName please raise an issue on github "+ str(response.status_code)
    else:
        data = "Error"
    return data


import pytest
from unittest.mock import Mock, patch
from nibss.credentials import Credentials
from nibss.bvn import Bvn
from nibss.records import Record
from nibss.fingerprint import FingerPrint
from nibss.tests.common import body, R, iv, aes, responses

b = {
    "base_url": "", "Organizationcode": "11111", "sandbox-key": "0hsagshfkdasjhkgfsdhlkjhgsvhgikjs",
    "content-type": "application/json", "accept": "application/json", "username": "11111", "password": "nbvhjfihkjsd"
}


@patch('requests.post')
def test_reset(mock_post):
    data = {'aes_key': aes, 'password': "fdgfudkjd", 'ivkey': iv}
    mock_post.return_value = R("")
    assert Credentials(b).reset() == data, "should return an object"

@patch('requests.post')
def test_verify_single(mock_post):
    data = responses["single_bvn"]
    mock_post.return_value = R(body["single_bvn"])
    assert Bvn(b).verify_single({"body": {"BVN": "12345678901"}, "Aes_key": aes, "Iv_key": iv}) == data, "should return object"

@patch('requests.post')
def test_multiple_bvn(mock_post):
    data = responses["multiple_bvn"]
    mock_post.return_value = R(body["multiple_bvn"])
    assert Bvn(b).verify_multiple({"bvns": {"BVNS": "1234567890 1, 12345678902, 12345678903"}, "Aes_key": aes, "Iv_key": iv}) == data, "should return object"

@patch('requests.post')
def test_watchlisted(mock_post):
    output = responses["watchlist"]
    mock_post.return_value = R(body["watchlist"])
    assert Bvn(b).bvn_watchlisted({"body": {"BVN": "12345678901"}, "Aes_key": aes, "Iv_key": iv}) == output, "should return an object"

@patch('requests.post')
def test_placeholder_reset(mock_post):
    data = {'aes_key': aes, 'password': "fdgfudkjd", 'ivkey': iv}
    mock_post.return_value = R("")
    assert Record(b).bvn_placeholder_reset() == data, "should return an object"

@patch('requests.post')
def test_validate_record(mock_post):
    response = responses["record"]
    mock_post.return_value = R(body["record"])
    assert Record(b).validate_record({
        "body": {
            "BVN": "12345678901",
            "FirstName": "Uchenna",
            "LastName": "Okoro",
            "MiddleName": "Adepoju",
            "AccountNumber": "0987654321",
            "BankCode": "011"
        },
        "Aes_key": aes, "Iv_key": iv}) == response, "should return an object"

@patch('requests.post')
def test_validate_records(mock_post):
    response = responses["records"]
    mock_post.return_value = R(body["records"])
    assert Record(b).validate_records({"body": [
        {
            "BVN": "12345678901",
            "FirstName": "Uchenna",
            "LastName": "Okoro",
            "MiddleName": "Adepoju",
            "AccountNumber": "0987654321",
            "BankCode": "011"
        },
        {
            "BVN": "12345678912",
            "FirstName": "Chidi",
            "LastName": "Seun",
            "MiddleName": "Joshua",
            "AccountNumber": "0987654329",
            "BankCode": "012"
        }
    ], "Aes_key": aes, "Iv_key": iv}) == response, "should return an object"

@patch('requests.post')
def test_verify_finger_print(mock_post):
    response = responses["fingerprint_data"]
    mock_post.return_value = R(body["fingerprint_data"])
    assert FingerPrint(b).verify_fingerprint({"body": {
        "BVN": "12345678901",
        "DeviceId": "Z000112BC12",
        "ReferenceNumber": "00099201710012205354422",
        "FingerImage": {
            "type": "ISO_2005",
            "position": "RT",
            "nist_impression_type": "0",
            "value": "c2RzZnNkZnNzZGY="
        }
    }, "Aes_key": aes, "Iv_key": iv}) == response, "should return an object"




from setuptools import setup, find_packages
from os import path

here = path.abspath(path.dirname(__file__))

setup(
    name="innovation-sandbox",
    version="1.1.0",
    description="Python package for Innovation Sandbox",
    long_description=open("README.rst").read(),
    packages=find_packages(),
    url = "https://github.com/enyata/innovation-sandbox-python",
    license='MIT',
    setup_requires=['wheel'],
    classifiers=[
        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: MIT License',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],

)



import requests
from nibss.utils import crypt as crypting
from nibss.utils.error_handler import error


def request(head, url):
    r = requests.post(url=url, headers=head)
    if r.status_code == 200:
        data = {
            "aes_key": r.headers['Aes_key'],
            "password": r.headers['Password'],
            "ivkey": r.headers['Ivkey']
        }
    else:
        data = error(r.status_code)
    return data


def encrypted_request(head, url, aes, iv, body):
    encrypted = crypting.Crypt().encrypt(body, aes, iv)
    try:
        r = requests.post(url=url, headers=head, data=encrypted)
        if r.status_code == 200:
            data = crypting.Crypt().decrypt(r.text, aes, iv)
        else:
            data = error(r.status_code)
        return data
    except requests.exceptions.RequestException as e:
        return e
        sys.exit(1)




from .utils import *
from .bvn import *
from .credentials import *
from .fingerprint import *
from .Nibss import *
from .records import *
from .tests import *

__all__ = [
    'bvn',
    'credentials',
    'fingerprint',
    'Nibss',
    'records'
]


from nibss.Nibss import Nibss
from urllib.parse import urljoin
from nibss.utils.calls import request, encrypted_request
from url import url as BASE_URL


class Record(Nibss):
    def bvn_placeholder_reset(self):
        headers = self.header
        URL = urljoin(BASE_URL(self.url), "/nibss/BVNPlaceHolder/Reset")
        return request(headers, URL)

    def validate_record(self, data):
        URL = urljoin(BASE_URL(self.url), "/nibss/BVNPlaceHolder/ValidateRecord")
        headers = self.headers
        body = data["body"]
        return encrypted_request(headers, URL, data["Aes_key"], data["Iv_key"], body)

    def validate_records(self, data):
        URL = urljoin(BASE_URL(self.url), "/nibss/BVNPlaceHolder/ValidateRecords")
        headers = self.headers
        body = data["body"]
        return encrypted_request(headers, URL, data["Aes_key"], data["Iv_key"], body)





from urllib.parse import urljoin
from url import url as BASE_URL
from sterling.Sterling import Sterling
from sterling.utils.calls import calls


class Transfer(Sterling):
    def InterbankNameEnquiry(self, query):
        url = urljoin(BASE_URL(self.url), "/sterling/TransferAPIs/api/Spay/InterbankNameEnquiry")
        header = self.headers
        apiResponse = calls("get", url, query, "", header)
        return apiResponse



from faker import Faker

fake = Faker().random_number()

body = {
    "Referenceid": str(fake),
    "RequestType": str(fake),
    "Translocation": str(fake),
    "SessionID": str(fake),
    "FromAccount": str(fake),
    "ToAccount": str(fake),
    "Amount": str(fake),
    "DestinationBankCode": str(fake),
    "NEResponse": str(fake),
    "BenefiName": str(fake),
    "PaymentReference": str(fake),
    "OriginatorAccountName": str(fake),
    "translocation": str(fake)
}

responses = {"enquiry": {
    "message": "OK",
    "data": {
        "message": "success",
        "response": "success",
        "data": {
            "AccountName": str(Faker().name),
            "sessionID": str(fake),
            "AccountNumber": str(fake),
            "status": str(fake),
            "BVN": str(fake),
            "ResponseText": None
        }
    }
},

    "transfer": {
        "message": "OK",
        "data": {
            "message": "success",
            "response": "success",
            "responsedate": "null",
            "data": {
                "ResponseText": "Your transaction has been submitted for processing.",
                "status": str(fake),
            }
        }
    }
}

query = {
    "Referenceid": str(fake),
    "RequestType": str(fake),
    "Translocation": str(fake),
    "ToAccount": str(fake),
    "destinationbankcode": str(fake)
}


class R:
    def __init__(self, text):
        self.status_code = 200
        self.text = text
